"""Airzone Local API Demo."""

from typing import Any

from aiohttp.web_response import Response
from helpers import api_json_response

from aioairzone.const import API_DATA


class AirzoneDemo:
    """Airzone Local API Demo."""

    def __init__(self) -> None:
        """Local API Demo init."""

    def data(self) -> dict[str, Any]:
        """Return Local API Demo data."""
        return {
            API_DATA: [
                {
                    "systemID": 1,
                    "zoneID": 1,
                    "name": "zona01",
                    "on": 1,
                    "coolsetpoint": 25,
                    "coolmaxtemp": 30,
                    "coolmintemp": 18,
                    "heatsetpoint": 22,
                    "heatmaxtemp": 30,
                    "heatmintemp": 15,
                    "maxTemp": 30,
                    "minTemp": 15,
                    "setpoint": 26,
                    "roomTemp": 20,
                    "modes": [1, 2, 3, 4],
                    "mode": 3,
                    "speeds": 5,
                    "speed": 2,
                    "speed_values": [0, 1, 2, 3, 4, 5],
                    "speed_type": 0,
                    "coldStages": 3,
                    "coldStage": 2,
                    "heatStages": 3,
                    "heatStage": 1,
                    "humidity": 43,
                    "units": 0,
                    "errors": [
                        {"Zone": "Error 3"},
                        {"Zone": "Error 4"},
                        {"Zone": "Error 5"},
                        {"Zone": "Error 6"},
                        {"Zone": "Error 7"},
                        {"Zone": "Error 8"},
                        {"Zone": "Presence alarm"},
                        {"Zone": "Window alarm"},
                        {"Zone": "Anti-freezing alarm"},
                        {"Zone": "Zone without thermostat"},
                        {"Zone": "Low battery"},
                        {"Zone": "Active dew"},
                        {"Zone": "Active dew protection"},
                        {"Zone": "F05-H"},
                        {"Zone": "F06-H"},
                        {"Zone": "F05-C"},
                        {"Zone": "F06-C"},
                        {"system": "Error 9"},
                        {"system": "Error 13"},
                        {"system": "Error 11"},
                        {"system": "Error 15"},
                        {"system": "Error 16"},
                        {"system": "Error C09"},
                        {"system": "Error C11"},
                        {"system": "IU error IU4"},
                        {"system": "Error IAQ1"},
                        {"system": "Error IAQ4"},
                        {"Zone": "Error IAQ2"},
                        {"Zone": "Error IAQ3"},
                    ],
                    "air_demand": 1,
                    "floor_demand": 1,
                    "aq_mode": 2,
                    "aq_quality": 2,
                    "aq_thrlow": 10,
                    "aq_thrhigh": 40,
                    "slats_vertical": 2,
                    "slats_horizontal": 3,
                    "slats_vswing": 1,
                    "slats_hswing": 0,
                    "antifreeze": 1,
                    "eco_adapt": "manual",
                }
            ]
        }

    async def post(self) -> Response:
        """POST Local API Demo."""
        return api_json_response(self.data())



"""Airzone Local API Version."""

from typing import Any

from aiohttp.web_response import Response
from helpers import api_json_response

from aioairzone.const import API_VERSION


class AirzoneVersion:
    """Airzone Local API Version."""

    def __init__(self, version: str) -> None:
        """Local API Version init."""
        self.version: str = version

    def data(self) -> dict[str, Any]:
        """Return Local API Version data."""
        return {
            API_VERSION: self.version,
        }

    async def post(self) -> Response:
        """POST Local API version."""
        return api_json_response(self.data())



"""Airzone Local API WebServer."""

import random
from typing import Any

from aiohttp.web_response import Response
from helpers import api_json_response

from aioairzone.const import (
    API_INTERFACE,
    API_MAC,
    API_WIFI,
    API_WIFI_CHANNEL,
    API_WIFI_QUALITY,
    API_WIFI_RSSI,
    API_WS_AZ,
    API_WS_FIRMWARE,
    API_WS_TYPE,
)


class AirzoneWebServer:
    """Airzone Local API WebServer."""

    def __init__(self, mac: str) -> None:
        """Local API WebServer init."""
        self.firmware: str = "3.44"
        self.interface: str = API_WIFI
        self.mac: str = mac
        self.type: str = API_WS_AZ
        self.wifi_channel: int = 6
        self.wifi_rssi: int = -50

    def wifi_quality(self) -> int:
        """Convert Wifi RSSI to Quality."""
        quality: float
        if self.wifi_rssi <= -100:
            quality = 0
        elif self.wifi_rssi >= -50:
            quality = 100
        else:
            quality = 2 * (self.wifi_rssi + 100)
            quality = round(quality / 10, 0) * 10
        return int(quality)

    def data(self) -> dict[str, Any]:
        """Return Local API Version data."""
        return {
            API_INTERFACE: API_WIFI,
            API_MAC: self.mac,
            API_WIFI_CHANNEL: self.wifi_channel,
            API_WIFI_QUALITY: self.wifi_quality(),
            API_WIFI_RSSI: self.wifi_rssi,
            API_WS_FIRMWARE: self.firmware,
            API_WS_TYPE: self.type,
        }

    def refresh(self) -> None:
        """Refresh Local API WebServer."""
        wifi_rssi = self.wifi_rssi + random.randrange(-10, 10, 1)
        if wifi_rssi <= -100:
            self.wifi_rssi = -100
        elif wifi_rssi >= -50:
            self.wifi_rssi = -50
        else:
            self.wifi_rssi = wifi_rssi

    async def post(self) -> Response:
        """POST Local API WebServer."""
        self.refresh()
        return api_json_response(self.data())



"""Airzone Local API Integration."""

from typing import Any

from aiohttp import web
from aiohttp.web_response import Response
from helpers import api_filter_dict, api_json_response

from aioairzone.const import API_DRIVER, API_SYSTEM_ID


class AirzoneIntegration:
    """Airzone Local API Integration."""

    def __init__(self, driver: str) -> None:
        """Local API Integration init."""
        self.driver: str = driver

    def data(self) -> dict[str, Any]:
        """Return Local API Integration data."""
        return {
            API_DRIVER: self.driver,
        }

    async def post(self, request: web.Request) -> Response:
        # pylint: disable=unused-argument
        """POST Local API Integration."""
        return api_json_response(self.data())

    async def put(self, request: web.Request) -> Response:
        """PUT Local API Integration."""
        data = await request.json()
        if isinstance(data, dict):
            keys = list(data) + [API_SYSTEM_ID]

            driver = data.get(API_DRIVER)
            if driver is not None:
                self.driver = data[API_DRIVER]

            return api_json_response(api_filter_dict(self.data(), keys))

        return api_json_response(self.data())



"""Airzone Local API Config."""

from aioairzone.const import DEFAULT_PORT, DEFAULT_SYSTEM_ID
from aioairzone.localapi import ConnectionOptions

AIRZONE_OPTIONS = ConnectionOptions(
    "192.168.1.25",
    DEFAULT_PORT,
    DEFAULT_SYSTEM_ID,
    True,
)



"""Airzone Local API simulation."""

from aiohttp import web
from aiohttp.web_request import Request
from aiohttp.web_response import Response
from demo import AirzoneDemo
from helpers import api_json_error
from hvac import AirzoneACSStatus, AirzoneHVAC
from integration import AirzoneIntegration
from version import AirzoneVersion
from webserver import AirzoneWebServer

from aioairzone.common import TemperatureUnit
from aioairzone.const import API_ERROR_METHOD_NOT_SUPPORTED, API_V1, DEFAULT_PORT

# Airzone device simulation


class Airzone:
    """Airzone Local API."""

    def __init__(self) -> None:
        """Local API Version init."""
        self.demo: AirzoneDemo = AirzoneDemo()
        self.hvac: AirzoneHVAC = AirzoneHVAC()
        self.integration: AirzoneIntegration = AirzoneIntegration("driver")
        self.version: AirzoneVersion = AirzoneVersion("1.64")
        self.webserver: AirzoneWebServer = AirzoneWebServer("11:22:33:44:55:66")


airzone = Airzone()
airzone.hvac.add_zone("Salón", 1, 1, TemperatureUnit.CELSIUS)
airzone.hvac.acs.set_status(AirzoneACSStatus.ENABLED)

# Airzone Local API simulation

routes = web.RouteTableDef()


@routes.post(f"/{API_V1}/demo")
async def demo_post_handler(request: Request) -> Response:
    # pylint: disable=unused-argument
    """POST /demo."""
    return await airzone.demo.post()


@routes.post(f"/{API_V1}/hvac")
async def hvac_post_handler(request: Request) -> Response:
    """POST /hvac."""
    return await airzone.hvac.post(request)


@routes.put(f"/{API_V1}/hvac")
async def hvac_put_handler(request: Request) -> Response:
    """PUT /hvac."""
    return await airzone.hvac.put(request)


@routes.post(f"/{API_V1}/integration")
async def integration_post_handler(request: Request) -> Response:
    """POST /integration."""
    return await airzone.integration.post(request)


@routes.put(f"/{API_V1}/integration")
async def integration_put_handler(request: Request) -> Response:
    """PUT /integration."""
    return await airzone.integration.put(request)


@routes.post(f"/{API_V1}/version")
async def version_post_handler(request: Request) -> Response:
    # pylint: disable=unused-argument
    """POST /version."""
    return await airzone.version.post()


@routes.post(f"/{API_V1}/webserver")
async def webserver_post_handler(request: Request) -> Response:
    # pylint: disable=unused-argument
    """POST /webserver."""
    return await airzone.webserver.post()


@routes.get("/{tail:.*}")
async def root_get_handler(request: Request) -> Response:
    """GET root."""
    # pylint: disable=unused-argument
    raise web.HTTPInternalServerError()


@routes.post("/{tail:.*}")
async def root_post_handler(request: Request) -> Response:
    # pylint: disable=unused-argument
    """POST root."""
    return api_json_error(API_ERROR_METHOD_NOT_SUPPORTED)


@routes.put("/{tail:.*}")
async def root_put_handler(request: Request) -> Response:
    # pylint: disable=unused-argument
    """PUT root."""
    return api_json_error(API_ERROR_METHOD_NOT_SUPPORTED)


app = web.Application()
app.add_routes(routes)
web.run_app(app, port=DEFAULT_PORT)



"""Airzone Local API Device."""

from __future__ import annotations

import logging
from typing import Any

from .common import EcoAdapt, QAdapt, SystemType, parse_bool, parse_int, parse_str
from .const import (
    API_ECO_ADAPT,
    API_ERRORS,
    API_MANUFACTURER,
    API_MC_CONNECTED,
    API_POWER,
    API_Q_ADAPT,
    API_SYSTEM_FIRMWARE,
    API_SYSTEM_TYPE,
    API_ZONE_ID,
    AZD_AVAILABLE,
    AZD_CLAMP_METER,
    AZD_ECO_ADAPT,
    AZD_ENERGY,
    AZD_ERRORS,
    AZD_FIRMWARE,
    AZD_FULL_NAME,
    AZD_ID,
    AZD_MANUFACTURER,
    AZD_MASTERS,
    AZD_MASTERS_SLAVES,
    AZD_MODEL,
    AZD_PROBLEMS,
    AZD_Q_ADAPT,
    AZD_SLAVES,
    ERROR_SYSTEM,
)
from .zone import Zone

_LOGGER = logging.getLogger(__name__)


class System:
    """Airzone System."""

    def __init__(self, system_id: int, zone_data: dict[str, Any]):
        """System init."""
        self.available: bool = True
        self.clamp_meter: bool | None = None
        self.eco_adapt: EcoAdapt | None = None
        self.energy: int | None = None
        self.errors: dict[int, list[str]] = {}
        self.id: int = system_id
        self.firmware: str | None = None
        self.manufacturer: str | None = None
        self.masters: dict[int, Zone] = {}
        self.q_adapt: QAdapt | None = None
        self.slaves: dict[int, Zone] = {}
        self.type: SystemType | None = None
        self.zones: dict[int, Zone] = {}

        self.update_zone_data(zone_data)

    def update_zone_data(self, zone_data: dict[str, Any]) -> None:
        """Update System data."""
        self.available = True

        zone_id = int(zone_data.get(API_ZONE_ID, 0))
        errors: list[dict[str, str]] | None = zone_data.get(API_ERRORS)
        if errors is not None and zone_id > 0:
            self.errors[zone_id] = []
            for error in errors:
                for key, val in error.items():
                    self.add_error(zone_id, val, key)

    def data(self) -> dict[str, Any]:
        """Return Airzone system data."""
        data: dict[str, Any] = {}

        data[AZD_AVAILABLE] = self.get_available()

        clamp_meter = self.get_clamp_meter()
        if clamp_meter is not None:
            data[AZD_CLAMP_METER] = clamp_meter

            if clamp_meter:
                energy = self.get_energy()
                if energy is not None:
                    data[AZD_ENERGY] = energy

        eco_adapt = self.get_eco_adapt()
        if eco_adapt is not None:
            data[AZD_ECO_ADAPT] = eco_adapt

        errors = self.get_errors()
        if len(errors) > 0:
            data[AZD_ERRORS] = errors

        firmware = self.get_firmware()
        if firmware is not None:
            data[AZD_FIRMWARE] = firmware

        full_name = self.get_full_name()
        if full_name is not None:
            data[AZD_FULL_NAME] = full_name

        data[AZD_ID] = self.get_id()

        manufacturer = self.get_manufacturer()
        if manufacturer is not None:
            data[AZD_MANUFACTURER] = manufacturer

        masters = self.get_masters()
        if masters is not None:
            data[AZD_MASTERS] = masters

        masters_slaves = self.get_masters_slaves()
        if masters_slaves is not None:
            data[AZD_MASTERS_SLAVES] = masters_slaves

        model = self.get_model()
        if model is not None:
            data[AZD_MODEL] = model

        data[AZD_PROBLEMS] = self.get_problems()

        q_adapt = self.get_qadapt()
        if q_adapt is not None:
            data[AZD_Q_ADAPT] = q_adapt

        slaves = self.get_slaves()
        if slaves is not None:
            data[AZD_SLAVES] = slaves

        return data

    def add_error(self, zone_id: int, error: str, error_id: str | None = None) -> None:
        """Add system error."""
        if error_id is not None:
            if (
                error_id.casefold() == ERROR_SYSTEM
                and error not in self.errors[zone_id]
            ):
                self.errors[zone_id] += [error]
        else:
            if error not in self.errors[zone_id]:
                self.errors[zone_id] += [error]

    def add_zone(self, zone: Zone) -> None:
        """Add zone to system."""
        zone_id = zone.get_id()
        if zone_id not in self.zones:
            self.zones[zone_id] = zone
        if zone.get_master():
            if zone_id not in self.masters:
                self.masters[zone_id] = zone
            if zone_id in self.slaves:
                self.slaves.pop(zone_id)
        else:
            if zone_id not in self.slaves:
                self.slaves[zone_id] = zone
            if zone_id in self.masters:
                self.masters.pop(zone_id)

    def get_available(self) -> bool:
        """Return availability."""
        return self.available

    def get_clamp_meter(self) -> bool | None:
        """Return system clamp meter connection."""
        return self.clamp_meter

    def get_eco_adapt(self) -> EcoAdapt | None:
        """Return system Eco Adapt."""
        return self.eco_adapt

    def get_energy(self) -> int | None:
        """Return system energy consumption."""
        return self.energy

    def get_errors(self) -> list[str]:
        """Return system errors."""
        errors: list[str] = []
        for zone_errors in self.errors.values():
            errors += zone_errors
        return errors

    def get_id(self) -> int:
        """Return system ID."""
        return self.id

    def get_firmware(self) -> str | None:
        """Return system firmware."""
        if self.firmware and "." not in self.firmware and len(self.firmware) > 2:
            return f"{self.firmware[0:1]}.{self.firmware[1:]}"
        return self.firmware

    def get_full_name(self) -> str:
        """Return full name."""
        return f"Airzone [{self.get_id()}] System"

    def get_manufacturer(self) -> str | None:
        """Return system manufacturer."""
        return self.manufacturer

    def get_masters(self) -> list[int] | None:
        """Return system master zones."""
        if len(self.masters) > 0:
            return list(self.masters.keys())
        return None

    def get_masters_slaves(self) -> dict[int, list[int]] | None:
        """Return system masters and slaves zones."""
        if len(self.masters) == 0:
            return None
        masters: dict[int, list[int]] = {}
        for master in self.masters.values():
            master_id = master.get_id()
            masters[master_id] = []
            for slave in self.slaves.values():
                slave_master = slave.get_master_zone()
                if slave_master is None or slave_master == master_id:
                    masters[master_id] += [slave.get_id()]
        return masters

    def get_model(self) -> str | None:
        """Return system model."""
        if self.type:
            return str(self.type)
        return None

    def get_problems(self) -> bool:
        """Return system problems."""
        for zone_errors in self.errors.values():
            if len(zone_errors) > 0:
                return True
        return False

    def get_qadapt(self) -> QAdapt | None:
        """Return system Q-Adapt."""
        return self.q_adapt

    def get_slave_master(self, slave: Zone) -> Zone | None:
        """Find the corresponding master from a slave."""
        master_id = slave.get_master_zone()
        if master_id is None:
            if len(self.masters) > 0:
                master = list(self.masters.values())[0]
            else:
                master = None
            if len(self.masters) > 1:
                _LOGGER.warning(
                    "multiple masters for %s -> %s",
                    slave.get_system_zone_id(),
                    list(self.masters.keys()),
                )
        else:
            master = self.masters.get(master_id)
        return master

    def get_slaves(self) -> list[int] | None:
        """Return system slave zones."""
        if len(self.slaves) > 0:
            return list(self.slaves.keys())
        return None

    def set_available(self, available: bool) -> None:
        """Set availability."""
        self.available = available

    def set_eco_adapt(self, eco_adapt: EcoAdapt | None) -> None:
        """Set system Eco Adapt."""
        self.eco_adapt = eco_adapt

    def set_master_param(self, master: int | None, key: str, value: Any) -> None:
        """Update master parameter by key and value."""
        if master is None or master == 0:
            self.set_param_all_zones(key, value)
        else:
            self.set_param_master_zone(master, key, value)

    def set_param(self, key: str, value: Any) -> None:
        """Update parameter by key and value."""
        if key == API_ECO_ADAPT:
            self.eco_adapt = EcoAdapt(value)
        elif key == API_Q_ADAPT:
            self.q_adapt = QAdapt(value)

    def set_param_all_zones(self, key: str, value: Any) -> None:
        """Update all zones parameter by key and value."""
        for zone in self.zones.values():
            zone.set_param(key, value)

    def set_param_master_zone(self, master: int | None, key: str, value: Any) -> None:
        """Update master zone parameter by key and value."""
        for zone in self.zones.values():
            if zone.get_master():
                if zone.get_id() == master:
                    zone.set_param(key, value)
            else:
                if zone.get_master_zone() == master:
                    zone.set_param(key, value)

    def update_data(self, data: dict[str, Any]) -> None:
        """Update system parameters by dict."""

        self.available = True

        mc_connected = parse_bool(data.get(API_MC_CONNECTED))
        if mc_connected is not None:
            self.clamp_meter = mc_connected

        errors: list[dict[str, str]] | None = data.get(API_ERRORS)
        if errors is not None:
            self.errors[0] = []
            for error in errors:
                for val in error.values():
                    self.add_error(0, val)

        manufacturer = parse_str(data.get(API_MANUFACTURER))
        if manufacturer is not None:
            self.manufacturer = manufacturer

        power = parse_int(data.get(API_POWER))
        if power is not None:
            self.energy = power

        system_firmware = parse_str(data.get(API_SYSTEM_FIRMWARE))
        if system_firmware is not None:
            self.firmware = system_firmware

        system_type = parse_int(data.get(API_SYSTEM_TYPE))
        if system_type is not None:
            self.type = SystemType(system_type)

        q_adapt = parse_int(data.get(API_Q_ADAPT))
        if q_adapt is not None:
            self.q_adapt = QAdapt(q_adapt)



"""Airzone Local API HVAC."""

from enum import IntEnum
import random
from typing import Any, cast

from aiohttp import web
from aiohttp.web_response import Response
from helpers import (
    api_filter_dict,
    api_json_error,
    api_json_response,
    celsius_to_fahrenheit,
)

from aioairzone.common import (
    OperationMode,
    QAdapt,
    SystemType,
    TemperatureUnit,
    get_system_zone_id,
)
from aioairzone.const import (
    API_ACS_MAX_TEMP,
    API_ACS_MIN_TEMP,
    API_ACS_ON,
    API_ACS_POWER_MODE,
    API_ACS_SET_POINT,
    API_ACS_TEMP,
    API_COOL_MAX_TEMP,
    API_COOL_MIN_TEMP,
    API_COOL_SET_POINT,
    API_DATA,
    API_DOUBLE_SET_POINT,
    API_ERROR_HOT_WATER_NOT_CONNECTED,
    API_ERROR_REQUEST_MALFORMED,
    API_ERROR_SYSTEM_ID_NOT_AVAILABLE,
    API_ERROR_SYSTEM_ID_NOT_PROVIDED,
    API_ERROR_SYSTEM_ID_OUT_RANGE,
    API_ERROR_ZONE_ID_NOT_AVAILABLE,
    API_ERRORS,
    API_HEAT_MAX_TEMP,
    API_HEAT_MIN_TEMP,
    API_HEAT_SET_POINT,
    API_HUMIDITY,
    API_MANUFACTURER,
    API_MAX_TEMP,
    API_MC_CONNECTED,
    API_MIN_TEMP,
    API_MODE,
    API_MODES,
    API_NAME,
    API_ON,
    API_POWER,
    API_Q_ADAPT,
    API_ROOM_TEMP,
    API_SET_POINT,
    API_SYSTEM_FIRMWARE,
    API_SYSTEM_ID,
    API_SYSTEM_TYPE,
    API_SYSTEMS,
    API_UNITS,
    API_ZONE_ID,
)


class AirzoneACSStatus(IntEnum):
    """Supported features of the Airzone Local API."""

    ENABLED = 0
    DISABLED = 1
    BOGUS = 2


class AirzoneACS:
    """Airzone Local API ACS."""

    def __init__(self) -> None:
        """Local API ACS init."""
        self.id: int = 0
        self.on: bool = True
        self.power: bool = False
        self.status: AirzoneACSStatus = AirzoneACSStatus.DISABLED
        self.temp: int = 40
        self.temp_max: int = 65
        self.temp_min: int = 25
        self.temp_set: int = 45

    def data(self) -> dict[str, Any]:
        """Return Local API ACS data."""
        if self.status == AirzoneACSStatus.BOGUS:
            return {
                API_ACS_MAX_TEMP: 0,
                API_ACS_MIN_TEMP: 0,
                API_ACS_ON: 0,
                API_ACS_POWER_MODE: 0,
                API_ACS_SET_POINT: 0,
                API_ACS_TEMP: 0,
                API_SYSTEM_ID: 0,
            }

        return {
            API_ACS_MAX_TEMP: self.temp_max,
            API_ACS_MIN_TEMP: self.temp_min,
            API_ACS_ON: int(self.on),
            API_ACS_POWER_MODE: int(self.power),
            API_ACS_SET_POINT: self.temp_set,
            API_ACS_TEMP: self.temp,
            API_SYSTEM_ID: self.id,
        }

    def refresh(self) -> None:
        """Refresh Local API ACS."""
        if self.status != AirzoneACSStatus.ENABLED:
            return

        temp = self.temp_set + (random.randrange(-100, 100, 1) / 10)
        if temp <= self.temp_min:
            temp = self.temp_min
        elif temp >= self.temp_max:
            temp = self.temp_max
        self.temp = int(temp)

    def set_status(self, status: AirzoneACSStatus) -> None:
        """Set Airzone ACS status."""
        self.status = status

    async def post(self) -> Response:
        """POST Local API ACS."""
        if self.status == AirzoneACSStatus.DISABLED:
            return api_json_error(API_ERROR_HOT_WATER_NOT_CONNECTED)

        self.refresh()
        return api_json_response(self.data())

    async def put(self, data: dict[str, Any]) -> Response:
        """PUT Local API ACS."""
        if self.status == AirzoneACSStatus.DISABLED:
            return api_json_error(API_ERROR_HOT_WATER_NOT_CONNECTED)

        keys = list(data) + [API_SYSTEM_ID]

        on = data.get(API_ACS_ON)
        if on is not None:
            self.on = bool(on)

        power = data.get(API_ACS_POWER_MODE)
        if power is not None:
            self.power = bool(power)

        temp_set = data.get(API_ACS_SET_POINT)
        if temp_set is not None:
            self.temp_set = int(temp_set)

        return api_json_response(api_filter_dict(self.data(), keys))


class AirzoneSystem:
    """Airzone Local API System."""

    def __init__(self, system_id: int):
        """Local API System init."""
        self.errors: list[dict[str, str]] = []
        self.firmware: str = "3.36"
        self.id: int = system_id
        self.mc_connected: bool = False
        self.manufacturer: str = "Python"
        self.power: int = 0
        self.q_adapt: QAdapt = QAdapt.UNKNOWN
        self.type: SystemType = SystemType.C3

    def data(self) -> dict[str, Any]:
        """Return Local API System data."""
        return {
            API_ERRORS: self.errors,
            API_MANUFACTURER: self.manufacturer,
            API_MC_CONNECTED: int(self.mc_connected),
            API_POWER: self.power,
            API_Q_ADAPT: self.q_adapt,
            API_SYSTEM_FIRMWARE: self.firmware,
            API_SYSTEM_ID: self.id,
            API_SYSTEM_TYPE: self.type.value,
        }

    def matches(self, system_id: int) -> bool:
        """Check if system matches param."""
        matches = False
        if system_id in (0, 127, self.id):
            matches = True
        return matches

    def refresh(self) -> None:
        """Refresh Local API System."""
        if self.mc_connected:
            self.power = random.randrange(0, 5, 1)

    def post(self) -> dict[str, Any]:
        """POST Local API System."""
        self.refresh()
        return self.data()

    def put(self) -> dict[str, Any]:
        """PUT Local API System."""
        return self.data()


class AirzoneZone:
    """Airzone Local API Zone."""

    def __init__(self, name: str, system_id: int, zone_id: int, units: TemperatureUnit):
        """Local API Zone init."""
        self.cool_temp_set: float = 22
        self.cool_temp_max: float = 30
        self.cool_temp_min: float = 18
        self.heat_temp_set: float = 21
        self.heat_temp_max: float = 30
        self.heat_temp_min: float = 15
        self.errors: list[dict[str, str]] = []
        self.humidity: int = 50
        self.id: int = zone_id
        self.mode: OperationMode = OperationMode.HEATING
        self.modes: list[OperationMode] = [
            OperationMode.COOLING,
            OperationMode.HEATING,
            OperationMode.AUTO,
        ]
        self.name: str = name
        self.on: bool = True
        self.system: int = system_id
        self.temp: float = 22.3
        self.temp_max: float = 30
        self.temp_min: float = 15
        self.temp_set: float = 21
        self.units: TemperatureUnit = units

    def conv_temp(self, temp: float | int) -> float | int:
        """Convert temperature to Fahrenheit if needed."""
        if self.units == TemperatureUnit.FAHRENHEIT:
            return celsius_to_fahrenheit(temp)
        return temp

    def get_modes(self) -> list[int]:
        """Get list of modes as integers."""
        modes: list[int] = []
        for mode in self.modes:
            modes += [int(mode)]
        return modes

    def data(self) -> dict[str, Any]:
        """Return Local API Zone data."""
        _data: dict[str, Any] = {
            API_COOL_SET_POINT: self.conv_temp(self.cool_temp_set),
            API_COOL_MAX_TEMP: self.conv_temp(self.cool_temp_max),
            API_COOL_MIN_TEMP: self.conv_temp(self.cool_temp_min),
            API_ERRORS: self.errors,
            API_HEAT_SET_POINT: self.conv_temp(self.heat_temp_set),
            API_HEAT_MAX_TEMP: self.conv_temp(self.heat_temp_max),
            API_HEAT_MIN_TEMP: self.conv_temp(self.heat_temp_min),
            API_HUMIDITY: self.humidity,
            API_MAX_TEMP: self.conv_temp(self.temp_max),
            API_MIN_TEMP: self.conv_temp(self.temp_min),
            API_MODE: self.mode,
            API_MODES: self.get_modes(),
            API_ON: int(self.on),
            API_ROOM_TEMP: self.conv_temp(self.temp),
            API_SET_POINT: self.conv_temp(self.temp_set),
            API_SYSTEM_ID: self.system,
            API_UNITS: int(self.units),
            API_ZONE_ID: self.id,
        }
        if len(self.name) > 0:
            _data[API_NAME] = self.name
        if OperationMode.AUTO in _data[API_MODES]:
            _data[API_DOUBLE_SET_POINT] = True
        return _data

    def matches(self, system_id: int, zone_id: int) -> bool:
        """Check if Zone matches params."""
        matches = False
        if system_id == 0:
            if zone_id == 0:
                matches = True
            elif zone_id == self.id:
                matches = True
        elif zone_id == 0:
            if system_id == self.system:
                matches = True
        elif system_id == self.system and zone_id == self.id:
            matches = True
        return matches

    def refresh(self) -> None:
        """Refresh Local API Zone."""
        self.humidity = 50 + random.randrange(-40, 40, 1)

        temp = self.temp_set + (random.randrange(-100, 100, 1) / 10)
        if temp <= self.temp_min:
            temp = self.temp_min
        elif temp >= self.temp_max:
            temp = self.temp_max
        self.temp = temp

    def post(self) -> dict[str, Any]:
        """POST Local API Zone."""
        self.refresh()
        return self.data()

    def put(self, data: dict[str, Any]) -> dict[str, Any]:
        """PUT Local API Zone."""
        keys = list(data) + [API_SYSTEM_ID, API_ZONE_ID]

        cool_temp_set = data.get(API_COOL_SET_POINT)
        if cool_temp_set is not None:
            self.cool_temp_set = self.conv_temp(float(cool_temp_set))

        heat_temp_set = data.get(API_HEAT_SET_POINT)
        if heat_temp_set is not None:
            self.heat_temp_set = self.conv_temp(float(heat_temp_set))

        mode = data.get(API_MODE)
        if mode is not None:
            self.mode = OperationMode(mode)

        on = data.get(API_ON)
        if on is not None:
            self.on = bool(on)

        temp_set = data.get(API_SET_POINT)
        if temp_set is not None:
            temp_set = float(temp_set)
            conv_temp_set = self.conv_temp(temp_set)
            self.temp_set = conv_temp_set
            if self.mode == OperationMode.AUTO:
                if temp_set >= self.temp:
                    self.heat_temp_set = conv_temp_set
                else:
                    self.cool_temp_set = conv_temp_set
            elif self.mode == OperationMode.COOLING:
                self.cool_temp_set = conv_temp_set
            elif self.mode == OperationMode.HEATING:
                self.heat_temp_set = conv_temp_set

        return cast(dict[str, Any], api_filter_dict(self.data(), keys))


class AirzoneHVAC:
    """Airzone Local API HVAC."""

    def __init__(self) -> None:
        """Local API HVAC init."""
        self.acs: AirzoneACS = AirzoneACS()
        self.systems: dict[int, AirzoneSystem] = {}
        self.zones: dict[str, AirzoneZone] = {}

    def add_zone(
        self, name: str, system_id: int, zone_id: int, units: TemperatureUnit
    ) -> bool:
        """Local API HVAC add Zone."""
        system_zone_id = get_system_zone_id(system_id, zone_id)
        if system_zone_id not in self.zones:
            if system_id not in self.systems:
                system = AirzoneSystem(system_id)
                self.systems[system_id] = system

            zone = AirzoneZone(name, system_id, zone_id, units)
            self.zones[system_zone_id] = zone

            return True
        return False

    def check_system(self, system_id: int) -> bool:
        """Check if System exists."""
        for zone in self.zones.values():
            if system_id in (0, 127, zone.system):
                return True
        return False

    def check_zone(self, system_id: int, zone_id: int) -> bool:
        """Check if Zone exists."""
        for zone in self.zones.values():
            if system_id in (0, zone.system):
                if zone_id in (0, zone.id):
                    return True
        return False

    def system_response(
        self, system_id: int, hvac_data: list[dict[str, Any]]
    ) -> Response:
        """Return HVAC System data."""
        if len(hvac_data) == 0:
            if not self.check_system(system_id):
                return api_json_error(API_ERROR_SYSTEM_ID_NOT_AVAILABLE)
            return api_json_error(API_ERROR_REQUEST_MALFORMED)
        return api_json_response({API_DATA: hvac_data})

    def post_system(self, system_id: int) -> Response:
        """POST Local API HVAC System."""
        hvac_data: list[dict[str, Any]] = []
        for system in self.systems.values():
            if system.matches(system_id):
                hvac_data += [system.post()]
        return self.system_response(system_id, hvac_data)

    def put_system(self, system_id: int) -> Response:
        """PUT Local API HVAC Zone."""
        hvac_data: list[dict[str, Any]] = []
        for system in self.systems.values():
            if system.matches(system_id):
                hvac_data += [system.put()]
        return self.system_response(system_id, hvac_data)

    def zone_response(
        self, system_id: int, zone_id: int, hvac_data: list[dict[str, Any]]
    ) -> Response:
        """Return HVAC Zone data."""
        if len(hvac_data) == 0:
            if not self.check_system(system_id):
                return api_json_error(API_ERROR_SYSTEM_ID_NOT_AVAILABLE)
            if not self.check_zone(system_id, zone_id):
                return api_json_error(API_ERROR_ZONE_ID_NOT_AVAILABLE)
            return api_json_error(API_ERROR_REQUEST_MALFORMED)
        if system_id == 0:
            response = {
                API_SYSTEMS: [
                    {
                        API_DATA: hvac_data,
                    }
                ]
            }
        else:
            response = {
                API_DATA: hvac_data,
            }
        return api_json_response(response)

    def post_zone(self, system_id: int, zone_id: int) -> Response:
        """POST Local API HVAC Zone."""
        hvac_data: list[dict[str, Any]] = []
        for zone in self.zones.values():
            if zone.matches(system_id, zone_id):
                hvac_data += [zone.post()]
        return self.zone_response(system_id, zone_id, hvac_data)

    def put_zone(self, system_id: int, zone_id: int, data: dict[str, Any]) -> Response:
        """PUT Local API HVAC Zone."""
        hvac_data: list[dict[str, Any]] = []
        for zone in self.zones.values():
            if zone.matches(system_id, zone_id):
                hvac_data += [zone.put(data)]
        return self.zone_response(system_id, zone_id, hvac_data)

    async def post(self, request: web.Request) -> Response:
        # pylint: disable=too-many-return-statements
        """POST Local API HVAC."""
        data = await request.json()
        if isinstance(data, dict):
            system = data.get(API_SYSTEM_ID)
            zone = data.get(API_ZONE_ID)
        else:
            system = None
            zone = None

        if system is None:
            return api_json_error(API_ERROR_SYSTEM_ID_NOT_PROVIDED)

        if system == 127:
            return self.post_system(system)

        if system == 0:
            if zone is None:
                return await self.acs.post()
            if 0 <= zone <= 32:
                return self.post_zone(system, zone)
            return api_json_error(API_ERROR_ZONE_ID_NOT_AVAILABLE)

        if 0 < system <= 32:
            if zone is None:
                return self.post_system(system)
            return self.post_zone(system, zone)

        return api_json_error(API_ERROR_SYSTEM_ID_OUT_RANGE)

    async def put(self, request: web.Request) -> Response:
        # pylint: disable=too-many-return-statements
        """PUT Local API HVAC."""
        data = await request.json()
        if isinstance(data, dict):
            system = data.get(API_SYSTEM_ID)
            zone = data.get(API_ZONE_ID)
        else:
            system = None
            zone = None

        if system is None:
            return api_json_error(API_ERROR_SYSTEM_ID_NOT_PROVIDED)

        if system == 127:
            return self.put_system(system)

        if system == 0:
            if zone is None:
                return await self.acs.put(data)
            if 0 <= zone <= 32:
                return self.put_zone(system, zone, data)
            return api_json_error(API_ERROR_ZONE_ID_NOT_AVAILABLE)

        if 0 < system <= 32:
            if zone is None:
                return self.put_system(system)
            return self.put_zone(system, zone, data)

        return api_json_error(API_ERROR_SYSTEM_ID_OUT_RANGE)



"""Airzone Local API Helpers."""

from collections.abc import Mapping
import json
from typing import Any

from aiohttp import web
from aiohttp.web_response import Response

from aioairzone.const import API_ERROR, API_ERRORS


def api_error_dict(error: str) -> dict[str, Any]:
    """Local API error dict."""
    return {
        API_ERRORS: [
            {
                API_ERROR: error,
            },
        ],
    }


def api_filter_dict(data: Any, keep: list[str]) -> Any:
    """Local API dict filter."""

    if not isinstance(data, (Mapping, list)):
        return data

    if isinstance(data, list):
        return [api_filter_dict(val, keep) for val in data]

    filtered = {**data}

    keys = list(filtered)
    for key in keys:
        if key not in keep:
            filtered.pop(key)
        elif isinstance(filtered[key], Mapping):
            filtered[key] = api_filter_dict(filtered[key], keep)
        elif isinstance(filtered[key], list):
            filtered[key] = [api_filter_dict(item, keep) for item in filtered[key]]

    return filtered


def api_json_dumps(obj: Any) -> Any:
    """Local API JSON dumps."""
    return json.dumps(obj, indent=4, sort_keys=True)


def api_json_error(error: str) -> Response:
    """Local API error."""
    return api_json_response(api_error_dict(error))


def api_json_response(data: dict[str, Any]) -> Response:
    """Return Local API error."""
    return web.json_response(data, dumps=api_json_dumps)


def celsius_to_fahrenheit(celsius: float | int) -> float | int:
    """Convert Celsius to Fahrenheit."""
    fahrenheit = (celsius * 9 / 5) + 32
    if isinstance(celsius, float):
        return fahrenheit
    return int(fahrenheit)



"""Airzone modify parameters example."""

import asyncio
import json
import time
import timeit

import _config
import aiohttp
from aiohttp.client_exceptions import ClientConnectorError

from aioairzone.const import API_MODE, API_Q_ADAPT, API_SYSTEM_ID, API_ZONE_ID
from aioairzone.exceptions import InvalidHost
from aioairzone.localapi import AirzoneLocalApi


async def main() -> None:
    """Airzone modify parameters example."""

    async with aiohttp.ClientSession() as aiohttp_session:
        airzone = AirzoneLocalApi(aiohttp_session, _config.AIRZONE_OPTIONS)
        try:
            validate_start = timeit.default_timer()
            airzone_mac = await airzone.validate()
            validate_end = timeit.default_timer()
            if airzone_mac is not None:
                print(f"Airzone WebServer: {airzone_mac}")
            print(f"Validate time: {validate_end - validate_start}")
            print("***")

            update_start = timeit.default_timer()
            await airzone.update()
            update_end = timeit.default_timer()
            print(json.dumps(airzone.data(), indent=4, sort_keys=True))
            print(f"Update time: {update_end - update_start}")
            print("***")

            modify_start = timeit.default_timer()
            await airzone.set_hvac_parameters(
                {
                    API_SYSTEM_ID: 1,
                    API_ZONE_ID: 3,
                    API_MODE: 1,
                }
            )
            modify_end = timeit.default_timer()
            print(json.dumps(airzone.data(), indent=4, sort_keys=True))
            print(f"Modify time: {modify_end - modify_start}")
            print("***")

            print("Sleeping...")
            time.sleep(3)
            print("***")

            update_start = timeit.default_timer()
            await airzone.update()
            update_end = timeit.default_timer()
            print(json.dumps(airzone.data(), indent=4, sort_keys=True))
            print(f"Update time: {update_end - update_start}")
            print("***")

            modify_start = timeit.default_timer()
            await airzone.set_sys_parameters(
                {
                    API_SYSTEM_ID: 1,
                    API_Q_ADAPT: 0,
                }
            )
            modify_end = timeit.default_timer()
            print(json.dumps(airzone.data(), indent=4, sort_keys=True))
            print(f"Modify time: {modify_end - modify_start}")
            print("***")

            print("Sleeping...")
            time.sleep(3)
            print("***")

            update_start = timeit.default_timer()
            await airzone.update()
            update_end = timeit.default_timer()
            print(json.dumps(airzone.data(), indent=4, sort_keys=True))
            print(f"Update time: {update_end - update_start}")
        except (ClientConnectorError, InvalidHost):
            print("Invalid host.")


if __name__ == "__main__":
    asyncio.run(main())
